@@ -6,7 +6,7 @@
 #   use warnings;
 # ----------------------------------------------------------------
     use vars qw( $VERSION );
-    $VERSION = "0.03";
+    $VERSION = "0.05";
 # ----------------------------------------------------------------
     my $XML_ENCODING = "UTF-8";
     my $INTERNAL_ENCODING = "UTF-8";
@@ -33,7 +33,7 @@ write xml as string from hash tree
     my $tree = { rss => { channel => { item => [ {
         title   => "The Perl Directory",
         link    => "http://www.perl.org/",
-    }, { 
+    }, {
         title   => "The Comprehensive Perl Archive Network",
         link    => "http://cpan.perl.org/",
     } ] } } };
@@ -51,7 +51,7 @@ get remote xml file with HTTP-GET and parse it into hash tree
 get remote xml file with HTTP-POST and parse it into hash tree
 
     use XML::TreePP;
-    my $tpp = XML::TreePP->new( array_element => [qw( item )] );
+    my $tpp = XML::TreePP->new( force_array => [qw( item )] );
     my $cgiurl = "http://search.hatena.ne.jp/keyword";
     my $keyword = "ajax";
     my $cgiquery = "mode=rss2&word=".$keyword;
@@ -64,8 +64,8 @@ get remote xml file with HTTP-POST and parse it into hash tree
 XML::TreePP module parses XML file and expand it for a hash tree.
 And also generate XML file from a hash tree.
 This is a pure Perl implementation.
-You can also download XML from remote web server 
-like XMLHttpRequest object at JavaScript language. 
+You can also download XML from remote web server
+like XMLHttpRequest object at JavaScript language.
 
 =head1 EXAMPLES
 
@@ -146,7 +146,7 @@ The result dumped is following:
         }
     };
 
-The special node name of "#text" is used because this elements 
+The special node name of "#text" is used because this elements
 has attribute(s) in addition to the text node.
 
 =head1 METHODS
@@ -156,30 +156,30 @@ has attribute(s) in addition to the text node.
 This method constructs a new "XML::TreePP" object.
 Optional attributes are following:
 
-=head3 output_encoding => "UTF-8",
+=head2 output_encoding => "UTF-8",
 
-You can define a encoding of xml file generated by write/writefile 
-methods. On Perl 5.8.x and later, you can select it from every 
-encodings supported by Encode.pm. On Perl 5.6.x or before with 
-Jcode.pm, you can use "Shift_JIS", "EUC-JP", "ISO-2022-JP" and 
+You can define a encoding of xml file generated by write/writefile
+methods. On Perl 5.8.x and later, you can select it from every
+encodings supported by Encode.pm. On Perl 5.6.x or before with
+Jcode.pm, you can use "Shift_JIS", "EUC-JP", "ISO-2022-JP" and
 "UTF-8". The default value is "UTF-8".
 
-=head3 array_element => [ "rdf:li", "item", "-xmlns" ],
+=head2 force_array => [ "rdf:li", "item", "-xmlns" ],
 
-This option allows you to specify a list of element names which 
+This option allows you to specify a list of element names which
 should always be forced into an array representation
-The default value is null, it means that context of the elements 
+The default value is null, it means that context of the elements
 will determine to make array or to keep it scalar.
 
-=head3 first_out => [ "link", "title", "-type" ],
+=head2 first_out => [ "link", "title", "-type" ],
 
-This option allows you to specify a list of element/attribute 
+This option allows you to specify a list of element/attribute
 names which should always appears at first on output XML code.
 The default value is null, it means alphabetical order is used.
 
-=head3 last_out => [ "items", "item", "entry" ],
+=head2 last_out => [ "items", "item", "entry" ],
 
-This option allows you to specify a list of element/attribute 
+This option allows you to specify a list of element/attribute
 names which should always appears at last on output XML code.
 
 =head2 $tree = $tpp->parse( $source );
@@ -194,11 +194,11 @@ The first argument is a filename.
 
 =head2 $tree = $tpp->parsefile( $method, $url, $body );
 
-This method receive a XML file from a remote server via HTTP and 
-returns a hash tree converted. 
+This method receive a XML file from a remote server via HTTP and
+returns a hash tree converted.
 This medhot requires LWP::UserAgent module.
 $method is a method of HTTP connection: GET/POST/PUT/DELETE
-$url is URI of a XML file. 
+$url is URI of a XML file.
 $body is request body when you use POST method.
 
 =head2 $source = $tpp->write( $tree, $encode );
@@ -219,7 +219,7 @@ http://www.kawa.net/works/perl/treepp/treepp-e.html
 
 =head1 COPYRIGHT AND LICENSE
 
-Copyright (c) 2006 Yusuke Kawasaki.  All rights reserved.  This program 
+Copyright (c) 2006 Yusuke Kawasaki.  All rights reserved.  This program
 is free software; you can redistribute it and/or modify it under the same
 terms as Perl itself.
 
@@ -316,7 +316,7 @@ sub parse {
     }
     my $flat = $self->xml_to_flat( $textref );
     my $tree = $self->flat_to_tree( $flat, "" );
-    $tree;
+    wantarray ? ( $tree, $$textref ) : $tree;
 }
 # ----------------------------------------------------------------
 sub xml_to_flat {
@@ -338,9 +338,9 @@ sub xml_to_flat {
         ))
         > ([^<]*)
     }sxg ) {
-        my( $ahead, $match, $typePI, $contPI, $typeCDATA, $contCDATA, 
+        my( $ahead, $match, $typePI, $contPI, $typeCDATA, $contCDATA,
             $typeDocT, $contDocT,
-            $typeCmnt, $contCmnt, $typeElem, $contElem, $follow ) = 
+            $typeCmnt, $contCmnt, $typeElem, $contElem, $follow ) =
             ( $1, $2, $3, $4, $5, $6, $7, $8, $9, $10, $11, $12, $13 );
         if ( $ahead ne "" ) {
             ## warn "Invalid header ignored: [$ahead]\n";
@@ -350,14 +350,14 @@ sub xml_to_flat {
         if ( $typeElem ) {                          ## Element
             my $node = {};
             if ( $contElem =~ s#^/## ) {
-                $node->{endTag} ++ 
+                $node->{endTag} ++
             } elsif ( $contElem =~ s#/$## ) {
                 # one line
             } else {
                 $node->{startTag} ++;
             }
             $node->{tagName} = $1 if ( $contElem =~ s#^(\S+)\s*## );
-            unless ( $node->{endTag} ) {            
+            unless ( $node->{endTag} ) {
                 my $attr = {};
                 while ( $contElem =~ m/([^\s\=\"\']+)=(?:(")(.*?)"|'(.*?)')/sg ) {
                     my $key = $1;
@@ -461,17 +461,17 @@ sub hash_to_xml {
         next unless ref $loopkey;
         foreach my $key ( grep {!/^-/} @$loopkey ) {
             my $val = $hash->{$key};
-            if ( UNIVERSAL::isa( $val, "ARRAY" ) ) {
+            if ( ! defined $val ) {
+                push( @$out, "<$key />" );
+            } elsif ( UNIVERSAL::isa( $val, "ARRAY" ) ) {
                 my $child = $self->array_to_xml( $key, $val );
                 push( @$out, $child );
             } elsif ( ref $val ) {
                 my $child = $self->hash_to_xml( $key, $val );
                 push( @$out, $child );
-            } elsif ( $val ne "" ) {
+            } else {
                 my $child = $self->scalar_to_xml( $key, $val );
                 push( @$out, $child );
-            } else {
-                push( @$out, "<$key />" );
             }
         }
         foreach my $key ( grep { /^-/} @$loopkey ) {
@@ -503,17 +503,17 @@ sub array_to_xml {
     my $array = shift;
     my $out = [];
     foreach my $val ( @$array ) {
-        if ( UNIVERSAL::isa( $val, "ARRAY" ) ) {
+        if ( ! defined $val ) {
+            push( @$out, "<$name />\n" );
+        } elsif ( UNIVERSAL::isa( $val, "ARRAY" ) ) {
             my $child = $self->array_to_xml( $name, $val );
             push( @$out, $child );
         } elsif ( ref $val ) {
             my $child = $self->hash_to_xml( $name, $val );
             push( @$out, $child );
-        } elsif ( $val ne "" ) {
+        } else {
             my $child = $self->scalar_to_xml( $name, $val );
             push( @$out, $child );
-        } else {
-            push( @$out, "<$name />\n" );
         }
     }
 #   s/^(\s*<)/  $1/mg foreach @$out;
@@ -566,8 +566,8 @@ sub encode_from_to {
     eval { require Encode; } unless defined $Encode::VERSION;
     if ( defined $Encode::VERSION ) {
         Encode::from_to( $$str, $from, $to, Encode::FB_XMLCREF() );
-    } elsif (( uc($from) eq "ISO-8859-1" || uc($from) eq "LATIN-1" ) &&
-               uc($to) eq "UTF-8" ) {
+    } elsif (( uc($from) eq "ISO-8859-1" || uc($from) eq "US-ASCII" ||
+               uc($from) eq "LATIN-1" ) && uc($to) eq "UTF-8" ) {
         &latin1_to_utf8( $str );
     } else {
         my $jfrom = &get_jcode_name( $from );
